@@ -170,11 +170,12 @@ class DatabaseInstance(object):
         self.instance_number = instance_number
         self.prompt = "%d:%s@%s> " % (self.instance_number, self.username, self.database)  
     sqlname = pyparsing.Word(pyparsing.alphas + '$_#%*', pyparsing.alphanums + '$_#%*')
+    typname = pyparsing.Word(pyparsing.alphas + ' ')
     ls_parser = ( (pyparsing.Optional(sqlname("owner") + "/") + 
-                   pyparsing.Optional(sqlname("type") + "/") + 
+                   pyparsing.Optional(typname("type") + "/") +
                    pyparsing.Optional(sqlname("name")) +
                    pyparsing.stringEnd ) 
-                   | ( pyparsing.Optional(sqlname("type") + "/") + 
+                   | ( pyparsing.Optional(typname("type") + "/") +
                        pyparsing.Optional(sqlname("owner") + ".") +
                        pyparsing.Optional(sqlname("name")) +
                        pyparsing.stringEnd ))
@@ -222,11 +223,12 @@ class DatabaseInstance(object):
                 selectors[selector + '_op'] = 'LIKE'
             else:
                 selectors[selector + '_op'] = '='
+    def default_owner(self):
+        return self.username
     def objects(self, target, opts):
         identifier = self.parse_identifier(target)
-        clauses = []
         if (identifier['owner'] == '%') and (not opts.all):
-            identifier['owner'] = self.username
+            identifier['owner'] = self.default_owner()
         self.set_operators(identifier)
         if hasattr(opts, 'reverse') and opts.reverse:
             identifier['sort_direction'] = 'DESC'
@@ -300,7 +302,7 @@ def connect(connstr):
 class MySQLInstance(DatabaseInstance):
     rdbms = 'mysql'
     default_port = 3306
-    paramstyle = 'qmark'
+    paramstyle = 'format'
     def set_defaults(self):
         self.port = self.default_port       
         self.hostname = 'localhost'
@@ -315,7 +317,9 @@ class MySQLInstance(DatabaseInstance):
     def bindVariables(self, binds):
         'Puts a tuple of (name, value) pairs into the bind format desired by MySQL'
         return (i[1] for i in binds)
-    column_qry = """SELECT c.table_schema, t.table_type, c.table_name, c.column_name      
+    def default_owner(self):
+        return self.database
+    column_qry = """SELECT c.table_schema, t.table_type, c.table_name, c.column_name
                     FROM   information_schema.columns c
                     JOIN   information_schema.tables t ON (c.table_schema = t.table_schema
                                                            AND c.table_name = t.table_name)
@@ -330,8 +334,9 @@ class MySQLInstance(DatabaseInstance):
                         FROM   information_schema.tables
                         WHERE  table_schema %(owner_op)s LOWER(%(owner)S)
                         AND    table_type %(type_op)s UPPER(%(type)S)
-                        AND    table_name %(name_op)s LOWER(%(name)S)
+                        AND    table_name %(name_op)s %(name)S
                         ORDER BY table_schema, table_type, table_name %(sort_direction)s"""
+    parameter_qry = """SHOW variables LIKE '%%%s%%'%s%s""" 
     gerald_types = {'TABLE': gerald.mysql_schema.Table,
                     'VIEW': gerald.mysql_schema.View,
                     'BASE TABLE': gerald.mysql_schema.Table,
@@ -346,9 +351,10 @@ class PostgresInstance(DatabaseInstance):
     paramstyle = 'format'
     def set_defaults(self):
         self.port = os.getenv('PGPORT') or self.default_port
-        self.database = os.getenv('ORACLE_SID')
-        self.hostname = os.getenv('PGHOST') or 'localhost'
-        self.username = os.getenv('USER')
+        self.database = os.getenv('PGDATABASE')
+        self.hostname = os.getenv('PGHOST') or os.getenv('PGHOSTADDR') or 'localhost'
+        self.username = os.getenv('PGUSER') or os.getenv('USER')
+        self.password = os.getenv('PGPASSWORD')
     def connect(self):
         self.connection = psycopg2.connect(host = self.hostname, user = self.username, 
                                  password = self.password, database = self.database,
@@ -375,6 +381,7 @@ class PostgresInstance(DatabaseInstance):
                     FROM   information_schema.routines r
                     WHERE  ( (r.routine_schema %(owner_op)s LOWER(%(owner)S)) OR (r.routine_schema = 'public') )
                     AND    LOWER(r.routine_definition) LIKE %(text)S"""
+    parameter_qry = """SELECT name, unit, setting FROM pg_settings WHERE name LIKE LOWER('%%%s%%')%s%s""" 
     gerald_types = {'BASE TABLE': gerald.postgres_schema.Table,
                     'VIEW': gerald.postgres_schema.View}
 
@@ -435,6 +442,16 @@ class OracleInstance(DatabaseInstance):
     tables_and_views_qry = """SELECT table_name
                               FROM   all_tables
                               WHERE  table_name LIKE UPPER(%(table_name)S)"""
+    parameter_qry = """SELECT name, 
+                                              CASE type WHEN 1 THEN 'BOOLEAN'
+                                                        WHEN 2 THEN 'STRING'
+                                                        WHEN 3 THEN 'INTEGER'
+                                                        WHEN 4 THEN 'PARAMETER FILE'
+                                                        WHEN 5 THEN 'RESERVED'
+                                                        WHEN 6 THEN 'BIG INTEGER' END type, 
+                                                        value 
+                                       FROM v$parameter 
+                                       WHERE name LIKE LOWER('%%%s%%')%s%s"""
     def source(self, target, opts):
         return self._source(target, opts)
     def bindSyntax(self, varname):
@@ -447,4 +464,7 @@ class OracleInstance(DatabaseInstance):
 
                 
 if __name__ == '__main__':
-    doctest.testmod()
+    opts = OptionTestDummy(password='password')
+    db = DatabaseInstance('oracle://system:twttatl@orcl', opts)
+    print list(db.findAll(''))
+    #doctest.testmod()
