@@ -24,7 +24,11 @@ or with a python-like shorthand
 - catherinedevlin.blogspot.com  May 31, 2006
 """
 import sys, os, re, sqlpython, pyparsing, re, completion
-import datetime, pickle, binascii, subprocess, time, itertools, hashlib
+import datetime, pickle, binascii, subprocess, time, itertools
+try:
+    from hashlib import md5
+except ImportError:
+    from md5 import new as md5
 import traceback, operator
 from cmd2 import Cmd, make_option, options, Statekeeper, Cmd2TestCase, options_defined
 import operator
@@ -44,88 +48,10 @@ try:
 except (RuntimeError, ImportError):
     pass
 
-queries = {
-'resolve': """
-SELECT object_type, object_name, owner FROM (
-SELECT object_type, object_name, user AS owner, 1 priority
-FROM   user_objects
-WHERE object_name = :objName
-UNION ALL
-SELECT ao.object_type, ao.object_name, ao.owner, 2 priority
-FROM    all_objects ao
-JOIN      user_synonyms us ON (us.table_owner = ao.owner AND us.table_name = ao.object_name)
-WHERE us.synonym_name = :objName
-AND   ao.object_type != 'SYNONYM'
-UNION ALL
-SELECT ao.object_type, ao.object_name, ao.owner, 3 priority
-FROM    all_objects ao
-JOIN      all_synonyms asyn ON (asyn.table_owner = ao.owner AND asyn.table_name = ao.object_name)
-WHERE asyn.synonym_name = :objName
-AND   ao.object_type != 'SYNONYM'
-AND      asyn.owner = 'PUBLIC'
-UNION ALL 
-SELECT 'DIRECTORY' object_type, dir.directory_name, dir.owner, 6 priority
-FROM   all_directories dir
-WHERE  dir.directory_name = :objName
-UNION ALL 
-SELECT 'DATABASE LINK' object_type, db_link, owner, 7 priority
-FROM   all_db_links dbl
-WHERE  dbl.db_link = :objName
-) ORDER BY priority ASC,
-           length(object_type) ASC,
-           object_type DESC""", # preference: PACKAGE before PACKAGE BODY, TABLE before INDEX
-'tabComments': """
-SELECT comments
-FROM    all_tab_comments
-WHERE owner = :owner
-AND      table_name = :table_name""",
-'colComments': """
-SELECT
-atc.column_name,
-acc.comments             
-FROM all_tab_columns atc
-JOIN all_col_comments acc ON (atc.owner = acc.owner and atc.table_name = acc.table_name and atc.column_name = acc.column_name)
-WHERE atc.table_name = :object_name
-AND      atc.owner = :owner
-ORDER BY atc.column_id;""",
-'oneColComments': """
-SELECT atc.column_name,
-acc.comments             
-FROM all_tab_columns atc
-JOIN all_col_comments acc ON (atc.owner = acc.owner and atc.table_name = acc.table_name and atc.column_name = acc.column_name)
-WHERE atc.table_name = :object_name
-AND      atc.owner = :owner
-AND      acc.column_name = :column_name
-ORDER BY atc.column_id;""",
-#thanks to Senora.pm for "refs"
-'refs': """
-NULL               referenced_by, 
-c2.table_name      references, 
-c1.constraint_name constraint
-FROM
-user_constraints c1,
-user_constraints c2
-WHERE
-c1.table_name = :object_name
-and c1.constraint_type ='R'
-and c1.r_constraint_name = c2.constraint_name
-and c1.r_owner = c2.owner
-and c1.owner = :owner
-UNION
-SELECT c1.table_name      referenced_by, 
-NULL               references, 
-c1.constraint_name constraint
-FROM
-user_constraints c1,
-user_constraints c2
-WHERE
-c2.table_name = :object_name
-and c1.constraint_type ='R'
-and c1.r_constraint_name = c2.constraint_name
-and c1.r_owner = c2.owner
-and c1.owner = :owner       
 """
-}
+need expanded Gerald support to work for mysql:
+do_comments
+"""
 
 class SoftwareSearcher(object):
     def __init__(self, softwareList, purpose):
@@ -314,7 +240,7 @@ class BlobDisplayer(object):
         self.url = ''
         if under_limit:
             self.blob = blob.read()
-            self.hashed = hashlib.md5(self.blob).hexdigest()
+            self.hashed = md5(self.blob).hexdigest()
             self.extension = imagedetect.extension_from_data(self.blob)
             if self.folder_ok():
                 self.file_name = '%s/%s%s' % (
@@ -783,7 +709,7 @@ class sqlpyPlus(sqlpython.sqlpython):
             selecttext = self.expandWildSql(arg)
         else:
             selecttext = arg
-        self.querytext = 'select ' + selecttext
+        self.querytext = '%s %s' % (arg.parsed.command, selecttext)
         if self.varsUsed:
             self.curs.execute(self.querytext, self.varsUsed)
         else: # this is an ugly workaround for the evil paramstyle curse upon DB-API2
@@ -905,52 +831,53 @@ class sqlpyPlus(sqlpython.sqlpython):
         except IndexError:
             newarg = ''
         return self.onecmd(self.parsed(shortcut + ' ' + newarg))
+   
+    def _param_select(self, arg, seekme):
+        seekme = seekme.replace('*','%').replace('?','_')
+        query = self.parsed(self.current_instance.parameter_qry % (seekme, arg.parsed.terminator or ';', arg.parsed.suffix))
+        return self.do_select(query)
     
     def do_show(self, arg):
         '''
         show                  - display value of all sqlpython parameters
         show (parameter name) - display value of a sqlpython parameter
-        show parameter (parameter name) - display value of an ORACLE parameter
+        show parameter (parameter name) - display value of a database parameter
         show err (object type/name)     - errors from latest PL/SQL object compilation.
         show all err (type/name)        - all compilation errors from the user's PL/SQL objects.
         show (index/schema/tablespace/trigger/view/constraint/comment) on (table)
         '''
-        if arg.startswith('param') and self.rdbms == 'oracle':
-            try:
-                paramname = arg.split()[1].lower()
-            except IndexError:
-                paramname = ''
-            self.onecmd(self.parsed("""SELECT name, 
-                                              CASE type WHEN 1 THEN 'BOOLEAN'
-                                                        WHEN 2 THEN 'STRING'
-                                                        WHEN 3 THEN 'INTEGER'
-                                                        WHEN 4 THEN 'PARAMETER FILE'
-                                                        WHEN 5 THEN 'RESERVED'
-                                                        WHEN 6 THEN 'BIG INTEGER' END type, 
-                                                        value 
-                                       FROM v$parameter 
-                                       WHERE name LIKE '%%%s%%';""" % paramname))
+        if not arg.strip():
+            return Cmd.do_show(self, arg)
         else:
             argpieces = arg.lower().split()
             argpieces = [a for a in argpieces if not a.startswith('-')]
-            try:
-                for (kwd, shortcut) in (
-                        ('ind', '\\di'), ('schema', '\\dn'), 
-                        ('tablesp', '\\db'), ('trig', '\\dg'), 
-                        ('view', '\\dv'), ('cons', '\\dc'),
-                        ('comm', '\\dd'), ('ref', 'ref')):
-                    if argpieces[0].lower().startswith(kwd):
-                        return self._show_shortcut(shortcut, argpieces)
-                if argpieces[0][:3] == 'err':
-                    return self._show_errors(all_users=False, limit=1, targets=argpieces[1:])
-                elif argpieces[0][:3] == 'tab':
-                    return self.do_ls('table/*')
-                elif (argpieces[0], argpieces[1][:3]) == ('all','err'):
-                    return self._show_errors(all_users=False, limit=None, targets=argpieces[2:])
-            except IndexError:
-                pass
+            for (kwd, shortcut) in (
+                    ('ind', '\\di'), ('schema', '\\dn'), 
+                    ('tablesp', '\\db'), ('trig', '\\dg'), 
+                    ('view', '\\dv'), ('cons', '\\dc'),
+                    ('comm', '\\dd'), ('ref', 'ref')):
+                if argpieces[0].lower().startswith(kwd):
+                    return self._show_shortcut(shortcut, argpieces)
+            if argpieces[0][:3] == 'err':
+                return self._show_errors(all_users=False, limit=1, targets=argpieces[1:])
+            elif argpieces[0][:3] == 'tab':
+                return self.do_ls('table/*')
+            elif (argpieces[0], argpieces[1:2][:3]) == ('all','err'):
+                return self._show_errors(all_users=False, limit=None, targets=argpieces[2:])
+            elif argpieces[0].startswith('variable') or argpieces[0].startswith('param'):
+                argpieces.pop(0)
+                if argpieces[:1] == ['like']:
+                    argpieces.pop(0)
+                if argpieces:
+                    target = argpieces[0]
+                else:
+                    target = '%'
+                return self._param_select(arg, target.strip("'"))
+        try:
             return Cmd.do_show(self, arg)
-            
+        except NotImplementedError:
+            return self._param_select(arg, arg)
+                       
     def _vc(self, arg, opts, program):
         if not os.path.exists('.%s' % program):
             create = raw_input('%s repository not yet in current directory (%s).  Create (y/N)? ' % 
@@ -1100,7 +1027,7 @@ class sqlpyPlus(sqlpython.sqlpython):
         for (owner, object_type, name) in self.current_instance.objects(arg, opts):
             obj = self.current_instance.object_metadata(owner, object_type, name)
             if hasattr(obj, 'comments'):
-                self.poutput('%s %s.%s' % object_type, owner, name)
+                self.poutput('%s %s.%s' % (object_type, owner, name))
                 self.poutput(obj.comments)
                 if hasattr(obj, 'columns'):
                     columns = obj.columns.values()
@@ -1112,28 +1039,6 @@ class sqlpyPlus(sqlpython.sqlpython):
                         else:
                             self.poutput(col['name'])
 
-    def _resolve(self, identifier):
-        parts = identifier.split('.')
-        if len(parts) == 2:
-            owner, object_name = parts
-            object_type = self.select_scalar_list('SELECT object_type FROM all_objects WHERE owner = :owner AND object_name = :object_name',
-                              {'owner': owner, 'object_name': object_name.upper()}
-                              )[0]
-        elif len(parts) == 1:
-            object_name = parts[0]
-            self._execute(queries['resolve'], {'objName':object_name.upper()})
-            object_type, object_name, owner = self.curs.fetchone()
-        return object_type, owner, object_name
-        
-    def resolve(self, identifier):
-        """Checks (my objects).name, (my synonyms).name, (public synonyms).name
-        to resolve a database object's name. """
-        try:
-            return self._resolve(identifier)
-        except (TypeError, IndexError):
-            self.pfeedback('Could not resolve object %s.' % identifier)
-            return '', '', ''
-        
     def spoolstop(self):
         if self.spoolFile:
             self.stdout = self.stdoutBeforeSpool
@@ -1228,13 +1133,16 @@ class sqlpyPlus(sqlpython.sqlpython):
             self.perror(self.do_psql.__doc__)
             
     def _do_dir(self, type, arg, opts):
-        self._do_ls("%s/%s%s%s" % (type, str(arg), arg.parsed.terminator, arg.parsed.suffix), opts)
+        self._do_ls("%%/%s/%s%s%s" % (type, str(arg), arg.parsed.terminator, arg.parsed.suffix), opts)
 
-   
+
+    table_type_name = {'postgres': 'base table', 'mysql': 'BASE TABLE',
+                       'oracle': 'TABLE' }
     @options(standard_options)
     def do__dir_tables(self, arg, opts):
         'Shortcut for ``ls table/``'
-        self._do_dir('table', arg, opts)
+        import pdb; pdb.set_trace()
+        self._do_dir(self.table_type_name[self.rdbms], arg, opts)
 
     @options(standard_options)
     def do__dir_views(self, arg, opts):
